(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*               Pierre Weis, projet Cristal, INRIA Rocquencourt       *)
(*                                                                     *)
(* Copyright 2001, 2004 Institut National de Recherche en Informatique *)
(* et en Automatique. All rights reserved. This file is distributed    *)
(* under the terms of the Q Public License version 1.0.                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: pipe.ml,v 1.2 2009/11/13 20:29:41 weis Exp $ *)

(* The pipe module, to launch two programs connected as in a pipe. *)

(* The shell version:

#!/bin/sh

exec ($1 | $2)

*)

(* The Caml version, using module Unix. *)
let connect_stdin proc (fdin, fdout) =
  Unix.close fdout;
  Unix.dup2 fdin Unix.stdin;
  Unix.close fdin;
  proc ()
;;

let connect_stdout proc (fdin, fdout) =
  Unix.close fdin;
  Unix.dup2 fdout Unix.stdout;
  Unix.close fdout;
  proc ()
;;

let pipe_connect proc1 proc2 =
  let p = Unix.pipe () in
  match Unix.fork () with
  | 0 -> connect_stdin proc2 p
  | _ -> connect_stdout proc1 p
;;

let launch prog () = Unix.execv prog [| prog |];;

let launch_connected_processes prog1 prog2 =
  pipe_connect (launch prog1) (launch prog2)
;;
