(***********************************************************************)
(*                                                                     *)
(*                           Objective Caml                            *)
(*                                                                     *)
(*               Pierre Weis, projet Cristal, INRIA Rocquencourt       *)
(*                                                                     *)
(*  Copyright 2007 Institut National de Recherche en Informatique et   *)
(*  en Automatique.  All rights reserved.  This file is distributed    *)
(*  only by permission.                                                *)
(*                                                                     *)
(***********************************************************************)

(* $Id: module_aggregate.ml,v 1.2 2009/11/14 12:04:03 weis Exp $ *)

module Make (AInfo : Module_types.INFO) = struct

  module Info = AInfo;;

  module Aggregate = struct

      type site = {
        path : Path.path;
        info : Info.info;
      }
      ;;

      let free_site path = { path = path; info = Info.get path; }
      ;;

      let print_site ppf site =
         Format.fprintf ppf "{ path = %a; info = %a; }"
            Path.print site.path Info.print site.info
      ;;

      let add i = function
        | [] -> [i]
        | n :: _ as l -> (n + i) :: l
      ;;

      let link site1 site2 =
        let p1 = add 1 site1.path in
        let p2 =  add 2 site1.path in
        free_site (p1 @ p2)
      ;;

  end
  ;;

end
;;
